#!/usr/sbin/dtrace -s
/*
 * dnlcsnoop.d - snoop DNLC activity.
 *               Written in DTrace (Solaris 10 3/05).
 *
 * 14-Jun-2005, ver 0.70
 *
 *
 * USAGE:	dnlcsnoop.d     # wait several seconds, then hit Ctrl-C
 *
 * FIELDS:
 *		PID             Process ID
 *		CMD          	Command name
 *		TIME         	Elapsed time for lookup, us
 *		HIT          	DNLC hit Y/N
 *		PATH         	Path for DNLC lookup
 *
 * Standard Disclaimer: This is freeware, use at your own risk.
 *
 * 27-Mar-2004	Brendan Gregg	Created this.
 * 14-Jun-2005	   "      "	Rewrote this a lot.
 */

#pragma D option quiet

/*
 * Print header
 */
dtrace:::BEGIN
{
	printf("%6s %-10s %5s %3s %s\n",
	    "PID", "CMD", "TIME", "HIT", "PATH");
}

/*
 * DNLC lookup
 */
fbt:genunix:dnlc_lookup:entry
{
	/* store path */
	self->path = stringof(args[0]->v_path);

	/* check for trailing "/" */
	this->len = strlen(self->path);
	self->join = *(char *)(args[0]->v_path + this->len - 1) == '/' ? 
	    "" : "/";

	/* store lookup name */
	self->name = stringof(arg1);

	/* store start time */
	self->start = timestamp;
}

/*
 * DNLC return
 */
fbt:genunix:dnlc_lookup:return
/self->start/
{
	/* calculate elapsed time */
	this->elapsed = (timestamp - self->start) / 1000;

	/* print output */
	printf("%6d %-10s %5d %3s %s%s%s\n",
	    pid, execname, this->elapsed, arg1 == 0 ? "N" : "Y", 
	    self->path, self->join, self->name);

	/* clear variables */
	self->path = 0;
	self->join = 0;
	self->name = 0;
	self->start = 0;
}
